/****************************************************************************************************************************
  Async_HelloServer.ino - Dead simple AsyncWebServer for Teensy41 QNEthernet

  For Teensy41 with QNEthernet

  AsyncWebServer_Teensy41 is a library for the Teensy41 with QNEthernet

  Based on and modified from ESPAsyncWebServer (https://github.com/me-no-dev/ESPAsyncWebServer)
  Built by Khoi Hoang https://github.com/khoih-prog/AsyncWebServer_Teensy41
  Licensed under GPLv3 license
 *****************************************************************************************************************************/
#if !( defined(CORE_TEENSY) && defined(__IMXRT1062__) && defined(ARDUINO_TEENSY41) )
	#error Only Teensy 4.1 supported
#endif

// Debug Level from 0 to 4
#define _TEENSY41_ASYNC_TCP_LOGLEVEL_       1
#define _AWS_TEENSY41_LOGLEVEL_             1

#define SHIELD_TYPE     "Teensy4.1 QNEthernet"

#if (_AWS_TEENSY41_LOGLEVEL_ > 3)
	#warning Using QNEthernet lib for Teensy 4.1. Must also use Teensy Packages Patch or error
#endif

#define USING_DHCP            true
//#define USING_DHCP            false

#if !USING_DHCP
	// Set the static IP address to use if the DHCP fails to assign
	IPAddress myIP(192, 168, 2, 222);
	IPAddress myNetmask(255, 255, 255, 0);
	IPAddress myGW(192, 168, 2, 1);
	//IPAddress mydnsServer(192, 168, 2, 1);
	IPAddress mydnsServer(8, 8, 8, 8);
#endif

#include "QNEthernet.h"       // https://github.com/ssilverman/QNEthernet
using namespace qindesign::network;

#include <AsyncWebServer_Teensy41.h>

AsyncWebServer    server(80);

const int led = 13;

void handleRoot(AsyncWebServerRequest *request)
{
	digitalWrite(led, 1);
	request->send(200, "text/plain", String("Hello from AsyncWebServer_Teensy41 on ") + BOARD_NAME );
	digitalWrite(led, 0);
}

void handleNotFound(AsyncWebServerRequest *request)
{
	digitalWrite(led, 1);
	String message = "File Not Found\n\n";

	message += "URI: ";
	//message += server.uri();
	message += request->url();
	message += "\nMethod: ";
	message += (request->method() == HTTP_GET) ? "GET" : "POST";
	message += "\nArguments: ";
	message += request->args();
	message += "\n";

	for (uint8_t i = 0; i < request->args(); i++)
	{
		message += " " + request->argName(i) + ": " + request->arg(i) + "\n";
	}

	request->send(404, "text/plain", message);
	digitalWrite(led, 0);
}

void setup()
{
	pinMode(led, OUTPUT);
	digitalWrite(led, 0);

	Serial.begin(115200);

	while (!Serial && millis() < 5000);

	delay(200);

	Serial.print("\nStart Async_HelloServer2 on ");
	Serial.print(BOARD_NAME);
	Serial.print(" with ");
	Serial.println(SHIELD_TYPE);
	Serial.println(ASYNC_WEBSERVER_TEENSY41_VERSION);

	delay(500);

#if USING_DHCP
	// Start the Ethernet connection, using DHCP
	Serial.print("Initialize Ethernet using DHCP => ");
	Ethernet.begin();
#else
	// Start the Ethernet connection, using static IP
	Serial.print("Initialize Ethernet using static IP => ");
	Ethernet.begin(myIP, myNetmask, myGW);
	Ethernet.setDNSServerIP(mydnsServer);
#endif

	if (!Ethernet.waitForLocalIP(5000))
	{
		Serial.println(F("Failed to configure Ethernet"));

		if (!Ethernet.linkStatus())
		{
			Serial.println(F("Ethernet cable is not connected."));
		}

		// Stay here forever
		while (true)
		{
			delay(1);
		}
	}
	else
	{
		Serial.print(F("Connected! IP address:"));
		Serial.println(Ethernet.localIP());
	}

#if USING_DHCP
	delay(1000);
#else
	delay(2000);
#endif

	server.on("/", HTTP_GET, [](AsyncWebServerRequest * request)
	{
		handleRoot(request);
	});

	server.on("/inline", [](AsyncWebServerRequest * request)
	{
		request->send(200, "text/plain", "This works as well");
	});

	server.on("/gif", [](AsyncWebServerRequest * request)
	{
		static const uint8_t gif[] =
		{
			0x47, 0x49, 0x46, 0x38, 0x37, 0x61, 0x10, 0x00, 0x10, 0x00, 0x80, 0x01,
			0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0x2c, 0x00, 0x00, 0x00, 0x00,
			0x10, 0x00, 0x10, 0x00, 0x00, 0x02, 0x19, 0x8c, 0x8f, 0xa9, 0xcb, 0x9d,
			0x00, 0x5f, 0x74, 0xb4, 0x56, 0xb0, 0xb0, 0xd2, 0xf2, 0x35, 0x1e, 0x4c,
			0x0c, 0x24, 0x5a, 0xe6, 0x89, 0xa6, 0x4d, 0x01, 0x00, 0x3b
		};

		char gif_colored[sizeof(gif)];

		memcpy(gif_colored, gif, sizeof(gif));

		// Set the background to a random set of colors
		gif_colored[16] = millis() % 256;
		gif_colored[17] = millis() % 256;
		gif_colored[18] = millis() % 256;

		request->send(200, (char *) "image/gif", gif_colored);
	});

	server.onNotFound(handleNotFound);

	server.begin();

	Serial.print("HTTP EthernetWebServer started @ IP : ");
	Serial.println(Ethernet.localIP());
}

void loop()
{
}
