/****************************************************************************************************************************
  Async_SimpleWebServer.ino - Dead simple AsyncWebServer for Teensy41 QNEthernet

  For Teensy41 with QNEthernet

  AsyncWebServer_Teensy41 is a library for the Teensy41 with QNEthernet

  Based on and modified from ESPAsyncWebServer (https://github.com/me-no-dev/ESPAsyncWebServer)
  Built by Khoi Hoang https://github.com/khoih-prog/AsyncWebServer_Teensy41
  Licensed under GPLv3 license
 *****************************************************************************************************************************/

//
// A simple server implementation showing how to:
//  * serve static messages
//  * read GET and POST parameters
//  * handle missing pages / 404s
//

#if !( defined(CORE_TEENSY) && defined(__IMXRT1062__) && defined(ARDUINO_TEENSY41) )
	#error Only Teensy 4.1 supported
#endif

// Debug Level from 0 to 4
#define _TEENSY41_ASYNC_TCP_LOGLEVEL_       1
#define _AWS_TEENSY41_LOGLEVEL_             1

#define SHIELD_TYPE     "Teensy4.1 QNEthernet"

#if (_AWS_TEENSY41_LOGLEVEL_ > 3)
	#warning Using QNEthernet lib for Teensy 4.1. Must also use Teensy Packages Patch or error
#endif

#define USING_DHCP            true
//#define USING_DHCP            false

#if !USING_DHCP
	// Set the static IP address to use if the DHCP fails to assign
	IPAddress myIP(192, 168, 2, 222);
	IPAddress myNetmask(255, 255, 255, 0);
	IPAddress myGW(192, 168, 2, 1);
	//IPAddress mydnsServer(192, 168, 2, 1);
	IPAddress mydnsServer(8, 8, 8, 8);
#endif

#include "QNEthernet.h"       // https://github.com/ssilverman/QNEthernet
using namespace qindesign::network;

#include <AsyncWebServer_Teensy41.h>

AsyncWebServer    server(80);

const char* PARAM_MESSAGE = "message";

void notFound(AsyncWebServerRequest *request)
{
	request->send(404, "text/plain", "Not found");
}

void setup()
{
	Serial.begin(115200);

	while (!Serial && millis() < 5000);

	Serial.print("\nStart Async_SimpleWebServer on ");
	Serial.print(BOARD_NAME);
	Serial.print(" with ");
	Serial.println(SHIELD_TYPE);
	Serial.println(ASYNC_WEBSERVER_TEENSY41_VERSION);

	delay(500);

#if USING_DHCP
	// Start the Ethernet connection, using DHCP
	Serial.print("Initialize Ethernet using DHCP => ");
	Ethernet.begin();
#else
	// Start the Ethernet connection, using static IP
	Serial.print("Initialize Ethernet using static IP => ");
	Ethernet.begin(myIP, myNetmask, myGW);
	Ethernet.setDNSServerIP(mydnsServer);
#endif

	if (!Ethernet.waitForLocalIP(5000))
	{
		Serial.println(F("Failed to configure Ethernet"));

		if (!Ethernet.linkStatus())
		{
			Serial.println(F("Ethernet cable is not connected."));
		}

		// Stay here forever
		while (true)
		{
			delay(1);
		}
	}
	else
	{
		Serial.print(F("Connected! IP address:"));
		Serial.println(Ethernet.localIP());
	}

#if USING_DHCP
	delay(1000);
#else
	delay(2000);
#endif

	server.on("/", HTTP_GET, [](AsyncWebServerRequest * request)
	{
		request->send(200, "text/plain", "Hello, world from Teensy 4.1 and QNEthernet");
	});

	// Send a GET request to <IP>/get?message=<message>
	server.on("/get", HTTP_GET, [] (AsyncWebServerRequest * request)
	{
		String message;

		if (request->hasParam(PARAM_MESSAGE))
		{
			message = request->getParam(PARAM_MESSAGE)->value();
		}
		else
		{
			message = "No message sent";
		}

		request->send(200, "text/plain", "Hello, GET: " + message);
	});

	// Send a POST request to <IP>/post with a form field message set to <message>
	server.on("/post", HTTP_POST, [](AsyncWebServerRequest * request)
	{
		String message;

		if (request->hasParam(PARAM_MESSAGE, true))
		{
			message = request->getParam(PARAM_MESSAGE, true)->value();
		}
		else
		{
			message = "No message sent";
		}

		request->send(200, "text/plain", "Hello, POST: " + message);
	});

	server.onNotFound(notFound);

	server.begin();

	Serial.print("Server started @ ");
	Serial.println(Ethernet.localIP());
}

void loop()
{
}
