/****************************************************************************************************************************
  WebClient.h - Dead simple AsyncWebServer for Teensy41 QNEthernet

  For Teensy41 with QNEthernet

  AsyncWebServer_Teensy41 is a library for the Teensy41 with QNEthernet

  Based on and modified from ESPAsyncWebServer (https://github.com/me-no-dev/ESPAsyncWebServer)
  Built by Khoi Hoang https://github.com/khoih-prog/AsyncWebServer_Teensy41
  Licensed under GPLv3 license
 *****************************************************************************************************************************/

#include "defines.h"

char server[] = "arduino.cc";

// Initialize the Web client object
EthernetClient client;

void setup()
{
	// Open serial communications and wait for port to open:
	Serial.begin(115200);

	while (!Serial && millis() < 5000);

	Serial.print("\nStart WebClient on ");
	Serial.print(BOARD_NAME);
	Serial.print(" with ");
	Serial.println(SHIELD_TYPE);
	Serial.println(ASYNC_WEBSERVER_TEENSY41_VERSION);

	delay(500);

#if USING_DHCP
	// Start the Ethernet connection, using DHCP
	Serial.print("Initialize Ethernet using DHCP => ");
	Ethernet.begin();
#else
	// Start the Ethernet connection, using static IP
	Serial.print("Initialize Ethernet using static IP => ");
	Ethernet.begin(myIP, myNetmask, myGW);
	Ethernet.setDNSServerIP(mydnsServer);
#endif

	if (!Ethernet.waitForLocalIP(5000))
	{
		Serial.println(F("Failed to configure Ethernet"));

		if (!Ethernet.linkStatus())
		{
			Serial.println(F("Ethernet cable is not connected."));
		}

		// Stay here forever
		while (true)
		{
			delay(1);
		}
	}
	else
	{
		Serial.print(F("Connected! IP address:"));
		Serial.println(Ethernet.localIP());
	}

#if USING_DHCP
	delay(1000);
#else
	delay(2000);
#endif

	Serial.println();
	Serial.println(F("Starting connection to server..."));

	// if you get a connection, report back via serial
	if (client.connect(server, 80))
	{
		Serial.println(F("Connected to server"));
		// Make a HTTP request
		client.println(F("GET /asciilogo.txt HTTP/1.1"));
		client.println(F("Host: arduino.cc"));
		client.println(F("Connection: close"));
		client.println();
	}
}

void printoutData(void)
{
	// if there are incoming bytes available
	// from the server, read them and print them
	while (client.available())
	{
		char c = client.read();
		Serial.write(c);
	}
}

void loop()
{
	printoutData();

	// if the server's disconnected, stop the client
	if (!client.connected())
	{
		Serial.println();
		Serial.println(F("Disconnecting from server..."));
		client.stop();

		// do nothing forevermore
		while (true);
	}
}
